﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace EAA
{
    public abstract class MathFunc
    {
        public abstract MathFunc RebuildFunc();
        public abstract float GetY(float x);
        public PointF MidPos { get; set; }
        public int XSize { get; set; }
        public float Multiplier { get; set; }
        public Pen Pen { get; set; }
        public int Invert { get; set; }
        

        public MathFunc(PointF midPos, int xSize, float multiplier)
        {
            this.MidPos = midPos;
            this.XSize = xSize;
            this.Multiplier = multiplier;
            this.Pen = Pens.Gray;
            this.Invert = 1;
        }

        public void DrawFunc(Graphics g)
        {
            for (int x = -this.XSize / 2; x < this.XSize / 2; x++)
            {
                PointF curPoint = new PointF(this.MidPos.X + x, this.MidPos.Y + this.GetY(x));
                PointF nextPoint = new PointF(this.MidPos.X + x+1, this.MidPos.Y + this.GetY(x+1));
                g.DrawLine(this.Pen, curPoint, nextPoint);
            }
            if (Properties.Settings.Default.showAxis)
            {
                Pen axisPen = new Pen(Brushes.Gray, 1);
                axisPen.EndCap = System.Drawing.Drawing2D.LineCap.ArrowAnchor;
                g.DrawLine(axisPen, this.MidPos.X + (-this.XSize / 2 - 5), this.MidPos.Y, this.MidPos.X + (this.XSize / 2 + 5), this.MidPos.Y);
                g.DrawLine(axisPen, this.MidPos.X, this.MidPos.Y + (this.XSize / 2 + 5), this.MidPos.X, this.MidPos.Y + (- this.XSize / 2 - 5));
            }
        }
        
    }

    public class Sine : MathFunc
    {
        public float Omega { get; set; }
        public Sine(PointF midPos, int xSize, float multiplier, float omega) : base(midPos, xSize, multiplier)
        {
            this.Omega = omega;
        }

        public override float GetY(float x)
        {
            return this.Invert * this.Multiplier * (float)Math.Sin(this.Omega * x);
        }

        public override MathFunc RebuildFunc()
        {
            return new Sine(this.MidPos, this.XSize, this.Multiplier, this.Omega);
        }
    }

    public class Exponential : MathFunc
    {
        public int Exponent { get; set; }
        public Exponential(PointF midPos, int xSize, float multiplier, int exponent) : base(midPos, xSize, multiplier)
        {
            this.Exponent = exponent;
        }

        public override float GetY(float x)
        {
            return this.Invert * -this.Multiplier * (float)Math.Pow(x, this.Exponent);
        }

        public override MathFunc RebuildFunc()
        {
            return new Exponential(this.MidPos, this.XSize, this.Multiplier, this.Exponent);
        }
    }

    public class Circle : MathFunc
    {
        public Circle(PointF midPos, int xSize, float multiplier) : base(midPos, xSize, multiplier) { }

        public override float GetY(float x)
        {
            return (float)(-this.Invert * Math.Sqrt(this.Multiplier * this.Multiplier - x * x));
        }
        public override MathFunc RebuildFunc()
        {
            return new Circle(this.MidPos, this.XSize, this.Multiplier);
        }
    }
    public class Linear : MathFunc
    {
        public Linear(PointF midPos, int xSize, float multiplier) : base(midPos, xSize, multiplier) { }
        public override float GetY(float x)
        {
            return this.Invert * -this.Multiplier * x;
        }
        public override MathFunc RebuildFunc()
        {
            return new Linear(this.MidPos, this.XSize, this.Multiplier);
        }
    }
}
